from dataharvest.purifier.baidu_purifier import BaiJiaHaoPurifier
from dataharvest.purifier.baidu_purifier import BaiduBaikePurifier
from dataharvest.purifier.bilibili_purifier import BilibiliPurifier
from dataharvest.purifier.common_purifier import CommonPurifier
from dataharvest.purifier.purifier import AutoPurifier, BasePurifier
from dataharvest.purifier.qqnew_purifier import QQNewPurifier
from dataharvest.purifier.sll_purifier import SllPurifier
from dataharvest.purifier.sobaike_purifier import SoBaikePurifier
from dataharvest.purifier.sogou_purifier import SogouBaiKePurifier
from dataharvest.purifier.sohu_purifier import SohuPurifier
from dataharvest.purifier.toutiao_purifier import ToutiaoPurifier
from dataharvest.purifier.wangyi_purifier import WangYiPurifier
from dataharvest.purifier.wechat_purifier import WechatPurifier

__all__ = ["BasePurifier", "CommonPurifier", "AutoPurifier", "BaiduBaikePurifier",
           "SohuPurifier", "SllPurifier", "BaiJiaHaoPurifier", "WangYiPurifier",
           "SogouBaiKePurifier", "BilibiliPurifier", "QQNewPurifier",
           "SoBaikePurifier", "ToutiaoPurifier", "WechatPurifier"]





from dataharvest.spider.common_spider import CommonSpider
from dataharvest.spider.toutiao_spider import ToutiaoSpider
from dataharvest.spider.spider import AutoSpider, BaseSpider

__all__ = ["AutoSpider", "CommonSpider", "BaseSpider", "ToutiaoSpider"]



import html2text

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier


class CommonPurifier(BasePurifier):
    index = 2 ** 16

    def __init__(self):
        h = html2text.HTML2Text()
        h.ignore_links = True
        h.body_width = 0
        h.single_line_break = True
        h.ignore_images = True
        self.converter = h

    def match(self, url: str) -> bool:
        return True

    def purify(self, doc: Document, converter=None, **kwargs) -> Document:
        if not converter:
            converter = self.converter

        page_content = converter.handle(doc.page_content)

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=page_content)



import re

import html2text
from parsel import Selector

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier


class BaiduBaikePurifier(BasePurifier):
    index = 100

    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0
        self.convertor.single_line_break = True
        self.convertor.ignore_images = True

    def match(self, url: str) -> bool:
        return "baike.baidu.com/item" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 小标签清洗
        selector.xpath('//div[starts-with(@class, "editLemma")]').drop()
        selector.xpath('//div[starts-with(@class, "lemmaPicture")]').drop()
        selector.xpath('//span[starts-with(@class,"ttsBtn")]').drop()
        selector.xpath('//span[starts-with(@class,"supWrap")]').drop()

        # 内容拼装
        title = selector.xpath('//h1[contains(@class, "J-lemma-title")]/text()')[0].get()
        clean_data = f"# {title}\n\n"
        # 描述
        if desc := selector.xpath('//div[starts-with(@class, "lemmaDescText")]/text()').get():
            clean_data += f"{desc}\n\n"
        # 简介
        lemma_summary_e = selector.xpath(
            '//div[starts-with(@class, "lemmaSummary") and contains(@class, "J-summary")]').get()
        summary = self.convertor.handle(lemma_summary_e)
        clean_data += f"## 简介\n\n{summary}"
        # 基本信息
        if item_elements := selector.xpath("//div[starts-with(@class, 'itemWrapper')]"):
            item_dict = {item.xpath("./dt/text()").get(): item.xpath("./dd/span/text()").get() for item in
                         item_elements}
            clean_data += "## 基本信息\n\n"
            clean_data += "\n".join([f"* {k}: {v}" for k, v in item_dict.items()])
        # 正文
        clean_data += "\n\n"
        if content_e := selector.xpath('//div[@class="J-lemma-content"]'):
            clean_data += self.convertor.handle(content_e[0].get())

        # 最终杂质字符
        clean_data = clean_data.replace("\u00A0", "")
        return Document(url=doc.url, metadata={**doc.metadata}, page_content=clean_data)


class BaiJiaHaoPurifier(BasePurifier):

    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "baijiahao.baidu.com/s" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清洗无用标签
        selector.xpath("//span[@data-testid='report-btn']").drop()

        title_text = selector.xpath("//div[@id='header']/div/text()").get()
        title = f"# {title_text}\n\n"

        content_label = selector.xpath("//div[@data-testid='article']").get()
        content_label_replaced = re.sub(r"<img\b[^>]*?>", "[图片]", content_label)

        content = self.convertor.handle(content_label_replaced)

        clean_data = title + content

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=clean_data)


import re

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier
import html2text
from parsel import Selector


class SohuPurifier(BasePurifier):

    def __init__(self):
        self.convertor = html2text.HTML2Text()

    def match(self, url: str) -> bool:
        return "www.sohu.com/a" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清洗无用的标签
        selector.xpath("//a[@id='backsohucom']").drop()
        selector.xpath("//p[@data-role='editor-name']").drop()

        # 标题
        title_label = selector.xpath("//div[@class='text-title']/h1").get()
        title = self.convertor.handle(title_label)

        # 正文
        article_label = selector.xpath("//article[@id='mp-editor']").get()

        article_label_replaced = re.sub(r"<img\b[^>]*?>", "[图片]", article_label)

        article = self.convertor.handle(article_label_replaced)

        clean_data = title + article

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=clean_data)



import re

import html2text
from parsel import Selector

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier


class BilibiliPurifier(BasePurifier):

    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "www.bilibili.com/read" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 标题
        title_label = selector.xpath("//h1[@class='title']").get()
        title = self.convertor.handle(title_label)

        # 内容
        content_label = selector.xpath("//div[@class='ql-editor']").get()
        content_label_replaced = re.sub(r"<img\b[^>]*?>", "[图片]", content_label)

        content = self.convertor.handle(content_label_replaced)

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=title + content)



from abc import ABC, abstractmethod

from dataharvest.base import Document


class BasePurifier(ABC):
    index = 0

    @abstractmethod
    def match(self, url: str) -> bool:
        raise NotImplementedError

    @abstractmethod
    def purify(self, doc: Document) -> Document:
        raise NotImplementedError


class AutoPurifier:
    _purifiers = []

    def __init__(self):
        AutoPurifier._purifiers = [cls() for cls in BasePurifier.__subclasses__()]
        AutoPurifier._purifiers.sort(key=lambda purifier: purifier.index)

    def _route(self, url: str) -> BasePurifier:
        for purifier in self._purifiers:
            if purifier.match(url):
                return purifier
        raise Exception(f"Cannot find purifier for url: {url}")

    def purify(self, doc: Document) -> Document:
        purifier = self._route(doc.url)
        return purifier.purify(doc)



from dataharvest.purifier.purifier import AutoPurifier
from dataharvest.spider.spider import AutoSpider


def test_auto_purifier():
    url = "https://zhuanlan.zhihu.com/p/369984873"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_sohu():
    url = "https://www.sohu.com/a/325718406_120013344"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_sll():
    url = "http://www.360doc.com/content/23/0613/11/72042116_1084562587.shtml"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_baijiahao():
    url = "https://baijiahao.baidu.com/s?id=1800439094856373024"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_wangyi():
    url = "https://www.163.com/auto/article/J6361L350008856R.html?clickfrom=w_lb_4_big"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_sogoubaike():
    url = "https://baike.sogou.com/v63038718.htm"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_zhihu():
    url = "https://zhuanlan.zhihu.com/p/706922799"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_bilibili():
    url = "https://www.bilibili.com/read/cv35655718/?from=category_0&jump_opus=1"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_qqnew():
    url = "https://new.qq.com/rain/a/20240703A09D9300"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_sobaike():
    url = "https://baike.so.com/doc/5579340-5792710.html?src=index#entry_concern"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_toutiao():
    url = "https://www.toutiao.com/article/7386866265849168419/?log_from=e75b95143ac23_1720001882524"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


def test_auto_purifier_wechat():
    url = "https://mp.weixin.qq.com/s/g7WfEc5UxAi3aMyDmwQnow"
    auto_spider = AutoSpider()
    doc = auto_spider.crawl(url)
    auto_purifier = AutoPurifier()
    doc = auto_purifier.purify(doc)
    print(doc)


import html2text
from parsel import Selector

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier


class SogouBaiKePurifier(BasePurifier):

    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "baike.sogou.com/v" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清理无用标签
        selector.xpath("//sup").drop()
        selector.xpath("//span[@class='base-info-card-value-more']").drop()
        selector.xpath("//a[@class='btn_edit']").drop()
        selector.xpath("//span[starts-with(@id, 'singlePicContainer')]").drop()
        selector.xpath("//span[starts-with(@id, 'commonAlbum')]").drop()

        # 标题
        title_label = selector.xpath("//div[@class='lemma_name']/h1").get()
        clean_data = self.convertor.handle(title_label)

        # 描述
        description_label = selector.xpath("//div[@class='abstract']").get()
        clean_data += self.convertor.handle(description_label)
        clean_data += "\r\n\r\n"

        # 基本信息
        base_info_dict = {}
        base_info_list = selector.xpath("//table[@class='abstract_list']")
        for base_info_item in base_info_list:
            cur_tr_list = base_info_item.xpath("./tbody/tr")

            for tr_item in cur_tr_list:
                key = tr_item.xpath("./th/text()").get()
                value = tr_item.xpath("./td//div[@class='base-info-card-value']/div/text()").get()

                base_info_dict[key] = value

        clean_data += "## 基本信息\r\n"
        clean_data += "\n".join([f"* {k}: {v}" for k, v in base_info_dict.items()])
        clean_data += "\r\n\r\n"

        # 正文
        content_label = selector.xpath("//div[@id='j-paragraph-wrap']").get()
        clean_data += self.convertor.handle(content_label)

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=clean_data)



from playwright.async_api import async_playwright
from playwright.sync_api import sync_playwright

from dataharvest.base import Document
from dataharvest.spider.spider import BaseSpider


class CommonSpider(BaseSpider):
    index = 2 ** 16

    def match(self, url: str) -> bool:
        return True

    def crawl(self, url: str):
        with sync_playwright() as playwright:
            browser = playwright.chromium.launch()
            page = browser.new_page()
            js = """
                    Object.defineProperties(navigator, {webdriver:{get:()=>undefined}});
                    """
            page.add_init_script(js)
            page.goto(url)
            page.wait_for_load_state('load')
            html = page.content()
            document = Document(url=page.url, metadata={}, page_content=html)
            return document

    async def a_crawl(self, url: str):
        async with async_playwright() as p:
            browser = await p.chromium.launch()
            page = await browser.new_page()
            js = """
                    Object.defineProperties(navigator, {webdriver:{get:()=>undefined}});
                    """
            await page.add_init_script(js)
            await page.goto(url)
            await page.wait_for_load_state('load')
            html = await page.content()
            await browser.close()
            return Document(url=url, metadata={}, page_content=html)



import cssutils
import html2text
from parsel import Selector

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier


class SoBaikePurifier(BasePurifier):

    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.ignore_links = True
        self.convertor.body_width = 0

    def match(self, url: str) -> bool:
        return "baike.so.com/doc/" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清理无用标签
        selector.xpath("//div[@id='main-content-text']//a[@class='conArrow']").drop()
        selector.xpath("//div[@id='main-content-text']//a[@class='reference_sup']").drop()
        selector.xpath("//div[@id='main-content-text']//span[@class='opt js-edittext']").drop()
        selector.xpath("//a[@class='show-img layoutright']").drop()

        # 标题
        title = selector.xpath("//div[@id='baike-title']/h1/span[@class='title']/text()").get()
        clean_data = f"# {title}\r\n"

        # 所属类别
        classify_key = selector.xpath("//div[@class='entry-classify-down js-classify-list']/span/text()").get()
        classify_val = selector.xpath("//div[@class='classify-txt js-classify-txt short']/text()").get()

        clean_data += classify_key
        clean_data += classify_val + "\r\n\r\n"

        # 描述
        description_label = selector.xpath("//div[@id='js-card-content']").get()
        clean_data += self.convertor.handle(description_label)
        clean_data += "\r\n\r\n"

        # 基本信息
        base_info_dict = {}
        base_info_ul_list = selector.xpath("//div[@id='basic-info']/div[@class='card-list-box']/ul")

        for base_info_ul_item in base_info_ul_list:

            cur_li_list = base_info_ul_item.xpath("./li")

            for li_item in cur_li_list:
                key = li_item.xpath("./div/p[starts-with(@class,'cardlist-name')]/@title").get()
                value = li_item.xpath("./div/p[starts-with(@class,'cardlist-value')]/@title").get()

                base_info_dict[key] = value

        clean_data += "## 基本信息\r\n"
        clean_data += "\n".join([f"* {k}: {v}" for k, v in base_info_dict.items()])
        clean_data += "\r\n\r\n"

        # 正文
        content_label = selector.xpath("//div[@id='main-content-text']")

        style_tags = selector.xpath("//style/text()").getall()

        sheet = cssutils.parseString("\n".join([t for t in style_tags]))

        not_visible_elements = set()
        for rule in sheet:
            if rule.type == rule.STYLE_RULE:

                selectors = rule.selectorText.split(", ")
                styles = rule.style.cssText
                if "visibility: hidden" in styles or "display: none" in styles:
                    not_visible_elements.update(selectors)

        for not_visible_item in not_visible_elements:
            content_label.xpath(".//*[@class='" + str(not_visible_item)[1:] + "']").drop()

        print(content_label.get())
        clean_data += self.convertor.handle(content_label.get())

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=clean_data)



import re

import html2text
from parsel import Selector

from dataharvest.base import Document
from dataharvest.purifier.purifier import BasePurifier


class SllPurifier(BasePurifier):

    def __init__(self):
        self.convertor = html2text.HTML2Text()
        self.convertor.single_line_break = True

    def match(self, url: str) -> bool:
        return "www.360doc.com/content" in url

    def purify(self, doc: Document) -> Document:
        selector = Selector(doc.page_content)

        # 清洗无用标签
        selector.xpath("//i[@id='ArticleReadingBtn']").drop()

        h1_title_label = selector.xpath("//h1[@id='titiletext']").get()
        h1_title = self.convertor.handle(h1_title_label)

        art_content_str = ""

        art_content_list = selector.xpath("//td[@id='artContent']/*").getall()
        for content_item in art_content_list:
            print(content_item)
            art_content_str += content_item

        content_label_replaced = re.sub(r"<img\b[^>]*?>", "[图片]", art_content_str)
        content = self.convertor.handle(content_label_replaced)

        clean_data = h1_title + content

        return Document(url=doc.url, metadata={**doc.metadata}, page_content=clean_data)



from abc import ABC, abstractmethod


class BaseSpider(ABC):
    index = 0

    @abstractmethod
    def match(self, url: str) -> bool:
        raise NotImplementedError

    @abstractmethod
    def crawl(self, url: str):
        raise NotImplementedError

    @abstractmethod
    async def a_crawl(self, url: str):
        raise NotImplementedError


class AutoSpider:
    _spiders = []

    def __init__(self):
        AutoSpider._spiders = [cls() for cls in BaseSpider.__subclasses__()]
        AutoSpider._spiders.sort(key=lambda spider: spider.index)

    def _route(self, url: str) -> BaseSpider:
        for spider in self._spiders:
            if spider.match(url=url):
                return spider
        raise Exception(f"Cannot find spider for url: {url}")

    def crawl(self, url: str):
        spider = self._route(url)
        return spider.crawl(url)

    async def a_crawl(self, url: str):
        spider = self._route(url)
        return await spider.a_crawl(url)



import pytest

from dataharvest.spider.common_spider import CommonSpider
from dataharvest.spider.spider import AutoSpider


def test_common_spider():
    spider = CommonSpider()
    doc = spider.crawl("https://baike.baidu.com/item/%E9%87%91%E7%BC%95%E7%8E%89%E8%A1%A3/617831?fr=ge_ala")
    print(doc)


@pytest.mark.asyncio
async def test_async_common_spider():
    spider = CommonSpider()
    doc = await spider.a_crawl("https://baike.baidu.com/item/%E9%87%91%E7%BC%95%E7%8E%89%E8%A1%A3/617831?fr=ge_ala")
    print(doc)


def test_auto_spider():
    spider = AutoSpider()
    doc = spider.crawl("https://zhuanlan.zhihu.com/p/369984873")
    print(doc)
